/*
 *  
 *  Copyright (c) 2004 Steve Slaven, All Rights Reserved.
 *  
 *  This program is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU General Public License as
 *  published by the Free Software Foundation; either version 2 of
 *  the License, or (at your option) any later version.
 *  
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 *  GNU General Public License for more details.
 *  
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 *  MA 02111-1307 USA
 *  
*/
package net.hoopajoo.cheeselab3;
import java.sql.*;

public class ImportPostgres {

    public static void main(String[] args) {
        Connection postgres, derby;
        try {
            Class.forName( "org.apache.derby.jdbc.EmbeddedDriver" );
            Class.forName( "org.postgresql.Driver" );
            postgres = DriverManager.getConnection( "jdbc:postgresql://localhost/cheeselab", "postgres", "postgres" );
            derby = DriverManager.getConnection( "jdbc:derby:cheesedb" );
        } catch( Exception e ) {
            e.printStackTrace();
            return;
        }

        // Create schema
        try {
            PreparedStatement select_rooms = postgres.prepareStatement(
                                                 "select *, (y * -1)+20000 as use_y," +
                                                 " case when exit_n = 0 then -1 when exit_n = -1 then 0 else exit_n end as use_exit_n," +
                                                 " case when exit_s = 0 then -1 when exit_s = -1 then 0 else exit_s end as use_exit_s," +
                                                 " case when exit_e = 0 then -1 when exit_e = -1 then 0 else exit_e end as use_exit_e," +
                                                 " case when exit_w = 0 then -1 when exit_w = -1 then 0 else exit_w end as use_exit_w," +
                                                 " case when exit_u = 0 then -1 when exit_u = -1 then 0 else exit_u end as use_exit_u," +
                                                 " case when exit_d = 0 then -1 when exit_d = -1 then 0 else exit_d end as use_exit_d," +
                                                 " 'sys' as userid, 1 as mtime from room_t" +
                                                 " order by roomid"
                                                 //+ " limit 100"
                                             );

            PreparedStatement insert_room = derby.prepareStatement(
                                                "insert into room_t ( name, edesc, notes, terrain, overlays," +
                                                "	exit_n, door_n, " +
                                                "	exit_s, door_s, " +
                                                "	exit_e, door_e, " +
                                                "	exit_w, door_w, " +
                                                "	exit_u, door_u, " +
                                                "	exit_d, door_d, " +
                                                "	x, y, z, mtime, userid )" +
                                                "values ( ?, ?, ?, ?, ?," +
                                                "	?, ?," +
                                                "	?, ?," +
                                                "	?, ?," +
                                                "	?, ?," +
                                                "	?, ?," +
                                                "	?, ?," +
                                                "	?, ?, ?, ?, ? )"
                                            );

            // room_map[postgres_id] = derbyid
            int[] room_map = new int[ 30000 ];
            for( int i = 0; i < room_map.length; i++ )
                room_map[ i ] = -9;

            //PreparedStatement save_xlate = derby.prepareStatement( "insert into xlate_t (oldid, newid) values (?,?)" );
            //PreparedStatement get_xlate = derby.prepareStatement( "select newid from xlate_t where oldid=?" );

            // This is the field map by count to the new derby db
            String[] fields = {
                                  "name", "description", "notes", "terrain", "overlays",
                                  "use_exit_n", "exit_n_door",
                                  "use_exit_s", "exit_s_door",
                                  "use_exit_e", "exit_e_door",
                                  "use_exit_w", "exit_w_door",
                                  "use_exit_u", "exit_u_door",
                                  "use_exit_d", "exit_d_door",
                                  "x", "use_y", "z", "mtime", "userid"
                              };

            // We will loop twice, once saving id maps and again to actually insert stuff
            // since we know it will number from 1 up
            ResultSet rooms = select_rooms.executeQuery();
            int cnt = 0;
            while( rooms.next() ) {
                cnt++;

                // Get last room and save it
                int newid = cnt;
                int oldid = rooms.getInt( "roomid" );

                println( "Mapping " + oldid + " to " + newid );

                room_map[ oldid ] = newid;
                /*
                save_xlate.setInt( 1, oldid );
                save_xlate.setInt( 2, newid );
                save_xlate.executeUpdate();
                */
            }

            rooms = select_rooms.executeQuery();
            cnt = 0;
            while( rooms.next() ) {
                cnt++;
                println( "Inserting record #" + cnt + " origid: " + rooms.getString( "roomid" ) );
                for( int i = 0; i < fields.length; i++ ) {
                    byte[] bytes = rooms.getBytes( fields[ i ] );
                    if( bytes == null )
                        bytes = new byte[ 0 ];

                    // Convert to unicode
                    String s = new String( bytes, "ISO-8859-1" );

                    // Fix exits if needed
                    if( fields[ i ].startsWith( "use_exit" ) ) {
                        // change s to new id
                        int oldid = Integer.parseInt( s );
                        if( oldid > 0 ) {
                            /*
                            get_xlate.setInt( 1, oldid );
                            ResultSet rs_new = get_xlate.executeQuery();
                            if( rs_new.next() ) {
                            	s = rs_new.getString( "newid" );
                            	println( "Translated " + oldid + " to " + s + " for " + fields[ i ] );
                            }else{
                            	println( "ERROR! unable to translate room " + oldid );
                            }
                            */

                            int newid = room_map[ oldid ];
                            if( newid == -9 ) {
                                println( "ERROR! unable to translate room " + oldid );
                            } else {
                                s = Integer.toString( newid );
                            }
                        }
                    }

                    insert_room.setString( i + 1, s );
                }
                insert_room.executeUpdate();
            }
        } catch( Exception e ) {
            e.printStackTrace();
        }
    }

    static void println( String s ) {
        System.out.println( s );
    }
}

